package org.eclipse.jetty.toolchain.version;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;

/**
 * Display help information on jetty-version-maven-plugin.<br/> Call <pre>  mvn jetty-version:help -Ddetail=true -Dgoal=&lt;goal-name&gt;</pre> to display parameter details.
 *
 * @version generated on Thu Oct 20 13:46:02 MST 2011
 * @author org.apache.maven.tools.plugin.generator.PluginHelpGenerator (version 2.8)
 * @goal help
 * @requiresProject false
 * @threadSafe
 */
public class HelpMojo
    extends AbstractMojo
{
    /**
     * If <code>true</code>, display all settable properties for each goal.
     * 
     * @parameter expression="${detail}" default-value="false"
     */
    private boolean detail;

    /**
     * The name of the goal for which to show help. If unspecified, all goals will be displayed.
     * 
     * @parameter expression="${goal}"
     */
    private java.lang.String goal;

    /**
     * The maximum length of a display line, should be positive.
     * 
     * @parameter expression="${lineLength}" default-value="80"
     */
    private int lineLength;

    /**
     * The number of spaces per indentation level, should be positive.
     * 
     * @parameter expression="${indentSize}" default-value="2"
     */
    private int indentSize;


    /** {@inheritDoc} */
    public void execute()
        throws MojoExecutionException
    {
        if ( lineLength <= 0 )
        {
            getLog().warn( "The parameter 'lineLength' should be positive, using '80' as default." );
            lineLength = 80;
        }
        if ( indentSize <= 0 )
        {
            getLog().warn( "The parameter 'indentSize' should be positive, using '2' as default." );
            indentSize = 2;
        }

        StringBuffer sb = new StringBuffer();

        append( sb, "org.eclipse.jetty.toolchain:jetty-version-maven-plugin:1.0.7", 0 );
        append( sb, "", 0 );

        append( sb, "Jetty:: Version Management Maven Plugin", 0 );
        append( sb, "Build / Release Plugin for working with versions in jetty maven builds", 1 );
        append( sb, "", 0 );

        if ( goal == null || goal.length() <= 0 )
        {
            append( sb, "This plugin has 3 goals:", 0 );
            append( sb, "", 0 );
        }

        if ( goal == null || goal.length() <= 0 || "attach-version-text".equals( goal ) )
        {
            append( sb, "jetty-version:attach-version-text", 0 );
            append( sb, "Attach the VERSION.txt to the project.\nWill only attach the VERSION.txt if it exists.\n", 1 );
            append( sb, "", 0 );
            if ( detail )
            {
                append( sb, "Available parameters:", 1 );
                append( sb, "", 0 );

                append( sb, "basedir", 2 );
                append( sb, "The project basedir.", 3 );
                append( sb, "Required: Yes", 3 );
                append( sb, "Expression: ${project.basedir}", 3 );
                append( sb, "", 0 );

                append( sb, "classifier", 2 );
                append( sb, "The classifier to use for attaching the generated VERSION.txt artifact", 3 );
                append( sb, "", 0 );

                append( sb, "type", 2 );
                append( sb, "The type to use for the attaching the generated VERSION.txt artifact", 3 );
                append( sb, "", 0 );

                append( sb, "versionTextInputFile (Default: ${project.basedir}/VERSION.txt)", 2 );
                append( sb, "The existing VERSION.txt file.", 3 );
                append( sb, "Expression: ${version.text.file}", 3 );
                append( sb, "", 0 );
            }
        }

        if ( goal == null || goal.length() <= 0 || "help".equals( goal ) )
        {
            append( sb, "jetty-version:help", 0 );
            append( sb, "Display help information on jetty-version-maven-plugin.\nCall\n\u00a0\u00a0mvn\u00a0jetty-version:help\u00a0-Ddetail=true\u00a0-Dgoal=<goal-name>\nto display parameter details.", 1 );
            append( sb, "", 0 );
            if ( detail )
            {
                append( sb, "Available parameters:", 1 );
                append( sb, "", 0 );

                append( sb, "detail (Default: false)", 2 );
                append( sb, "If true, display all settable properties for each goal.", 3 );
                append( sb, "Expression: ${detail}", 3 );
                append( sb, "", 0 );

                append( sb, "goal", 2 );
                append( sb, "The name of the goal for which to show help. If unspecified, all goals will be displayed.", 3 );
                append( sb, "Expression: ${goal}", 3 );
                append( sb, "", 0 );

                append( sb, "indentSize (Default: 2)", 2 );
                append( sb, "The number of spaces per indentation level, should be positive.", 3 );
                append( sb, "Expression: ${indentSize}", 3 );
                append( sb, "", 0 );

                append( sb, "lineLength (Default: 80)", 2 );
                append( sb, "The maximum length of a display line, should be positive.", 3 );
                append( sb, "Expression: ${lineLength}", 3 );
                append( sb, "", 0 );
            }
        }

        if ( goal == null || goal.length() <= 0 || "update-version-text".equals( goal ) )
        {
            append( sb, "jetty-version:update-version-text", 0 );
            append( sb, "Update the active version entry in the VERSION.txt file from information present in the git logs.", 1 );
            append( sb, "", 0 );
            if ( detail )
            {
                append( sb, "Available parameters:", 1 );
                append( sb, "", 0 );

                append( sb, "attachArtifact (Default: false)", 2 );
                append( sb, "Allow the plugin to attach the generated VERSION.txt file to the project", 3 );
                append( sb, "Expression: ${version.attach}", 3 );
                append( sb, "", 0 );

                append( sb, "basedir", 2 );
                append( sb, "The project basedir.", 3 );
                append( sb, "Required: Yes", 3 );
                append( sb, "Expression: ${project.basedir}", 3 );
                append( sb, "", 0 );

                append( sb, "classifier", 2 );
                append( sb, "The classifier to use for attaching the generated VERSION.txt artifact", 3 );
                append( sb, "", 0 );

                append( sb, "copyGenerated (Default: false)", 2 );
                append( sb, "Allow the plugin to replace the input VERSION.txt file", 3 );
                append( sb, "Expression: ${version.copy.generated}", 3 );
                append( sb, "", 0 );

                append( sb, "refreshTags (Default: false)", 2 );
                append( sb, "Allow the plugin to issue a \'git fetch --tags\' to update the local tags from.", 3 );
                append( sb, "Expression: ${version.refresh.tags}", 3 );
                append( sb, "", 0 );

                append( sb, "sortExisting (Default: false)", 2 );
                append( sb, "Allow the existing issues to be sorted alphabetically.", 3 );
                append( sb, "Expression: ${version.sort.existing}", 3 );
                append( sb, "", 0 );

                append( sb, "type", 2 );
                append( sb, "The type to use for the attaching the generated VERSION.txt artifact", 3 );
                append( sb, "", 0 );

                append( sb, "updateDate (Default: false)", 2 );
                append( sb, "Allow the plugin to update the release date for an issue (if none is provided)", 3 );
                append( sb, "Expression: ${version.update.date}", 3 );
                append( sb, "", 0 );

                append( sb, "version (Default: ${project.version})", 2 );
                append( sb, "The maven project version.", 3 );
                append( sb, "Required: Yes", 3 );
                append( sb, "Expression: ${version.section}", 3 );
                append( sb, "", 0 );

                append( sb, "versionTagKey (Default: jetty-VERSION)", 2 );
                append( sb, "The version key to use when looking up a git tag ref.", 3 );
                append( sb, "Required: Yes", 3 );
                append( sb, "Expression: ${version.tag.key}", 3 );
                append( sb, "", 0 );

                append( sb, "versionTextInputFile (Default: ${project.basedir}/VERSION.txt)", 2 );
                append( sb, "The existing VERSION.txt file.", 3 );
                append( sb, "Expression: ${version.text.file}", 3 );
                append( sb, "", 0 );

                append( sb, "versionTextKey (Default: jetty-VERSION)", 2 );
                append( sb, "The version key to use in the VERSION.txt file.", 3 );
                append( sb, "Required: Yes", 3 );
                append( sb, "Expression: ${version.text.key}", 3 );
                append( sb, "", 0 );

                append( sb, "versionTextOuputFile (Default: ${project.build.directory}/VERSION.txt)", 2 );
                append( sb, "The generated VERSION.txt file.", 3 );
                append( sb, "Expression: ${version.text.output.file}", 3 );
                append( sb, "", 0 );
            }
        }

        if ( getLog().isInfoEnabled() )
        {
            getLog().info( sb.toString() );
        }
    }

    /**
     * <p>Repeat a String <code>n</code> times to form a new string.</p>
     *
     * @param str String to repeat
     * @param repeat number of times to repeat str
     * @return String with repeated String
     * @throws NegativeArraySizeException if <code>repeat < 0</code>
     * @throws NullPointerException if str is <code>null</code>
     */
    private static String repeat( String str, int repeat )
    {
        StringBuffer buffer = new StringBuffer( repeat * str.length() );

        for ( int i = 0; i < repeat; i++ )
        {
            buffer.append( str );
        }

        return buffer.toString();
    }

    /** 
     * Append a description to the buffer by respecting the indentSize and lineLength parameters.
     * <b>Note</b>: The last character is always a new line.
     * 
     * @param sb The buffer to append the description, not <code>null</code>.
     * @param description The description, not <code>null</code>.
     * @param indent The base indentation level of each line, must not be negative.
     */
    private void append( StringBuffer sb, String description, int indent )
    {
        for ( Iterator it = toLines( description, indent, indentSize, lineLength ).iterator(); it.hasNext(); )
        {
            sb.append( it.next().toString() ).append( '\n' );
        }
    }

    /** 
     * Splits the specified text into lines of convenient display length.
     * 
     * @param text The text to split into lines, must not be <code>null</code>.
     * @param indent The base indentation level of each line, must not be negative.
     * @param indentSize The size of each indentation, must not be negative.
     * @param lineLength The length of the line, must not be negative.
     * @return The sequence of display lines, never <code>null</code>.
     * @throws NegativeArraySizeException if <code>indent < 0</code>
     */
    private static List toLines( String text, int indent, int indentSize, int lineLength )
    {
        List lines = new ArrayList();

        String ind = repeat( "\t", indent );
        String[] plainLines = text.split( "(\r\n)|(\r)|(\n)" );
        for ( int i = 0; i < plainLines.length; i++ )
        {
            toLines( lines, ind + plainLines[i], indentSize, lineLength );
        }

        return lines;
    }

    /** 
     * Adds the specified line to the output sequence, performing line wrapping if necessary.
     * 
     * @param lines The sequence of display lines, must not be <code>null</code>.
     * @param line The line to add, must not be <code>null</code>.
     * @param indentSize The size of each indentation, must not be negative.
     * @param lineLength The length of the line, must not be negative.
     */
    private static void toLines( List lines, String line, int indentSize, int lineLength )
    {
        int lineIndent = getIndentLevel( line );
        StringBuffer buf = new StringBuffer( 256 );
        String[] tokens = line.split( " +" );
        for ( int i = 0; i < tokens.length; i++ )
        {
            String token = tokens[i];
            if ( i > 0 )
            {
                if ( buf.length() + token.length() >= lineLength )
                {
                    lines.add( buf.toString() );
                    buf.setLength( 0 );
                    buf.append( repeat( " ", lineIndent * indentSize ) );
                }
                else
                {
                    buf.append( ' ' );
                }
            }
            for ( int j = 0; j < token.length(); j++ )
            {
                char c = token.charAt( j );
                if ( c == '\t' )
                {
                    buf.append( repeat( " ", indentSize - buf.length() % indentSize ) );
                }
                else if ( c == '\u00A0' )
                {
                    buf.append( ' ' );
                }
                else
                {
                    buf.append( c );
                }
            }
        }
        lines.add( buf.toString() );
    }

    /** 
     * Gets the indentation level of the specified line.
     * 
     * @param line The line whose indentation level should be retrieved, must not be <code>null</code>.
     * @return The indentation level of the line.
     */
    private static int getIndentLevel( String line )
    {
        int level = 0;
        for ( int i = 0; i < line.length() && line.charAt( i ) == '\t'; i++ )
        {
            level++;
        }
        for ( int i = level + 1; i <= level + 4 && i < line.length(); i++ )
        {
            if ( line.charAt( i ) == '\t' )
            {
                level++;
                break;
            }
        }
        return level;
    }
}
